/******************************************************************************/
/* Copyright 2008-2009 CNRS, INRIA                                            */
/*                                                                            */
/* File          : hips_wrapper.h                                             */
/*                                                                            */
/* Description   : This file is part of HIPS : Hierarchical Iterative         */
/*                 Parallel Solver (http://hips.gforge.inria.fr/).            */
/*                                                                            */
/* Authors       : J. GAIDAMOUR and P. HENON                                  */
/******************************************************************************/

/******************************************************************************/
/* License       : CeCILL-C                                                   */
/*                                                                            */
/* This software is governed by the CeCILL-C license under French law and     */
/* abiding by the rules of distribution of free software. You can use,        */
/* modify and/or redistribute the software under the terms of the             */
/* CeCILL-C license as circulated by CEA, CNRS and INRIA at the following     */
/* URL: "http://www.cecill.info".                                             */
/*                                                                            */
/* As a counterpart to the access to the source code and rights to copy,      */
/* modify and redistribute granted by the license, users are provided         */
/* only with a limited warranty and the software's author, the holder of      */
/* the economic rights, and the successive licensors have only limited        */
/* liability.                                                                 */
/*                                                                            */
/* In this respect, the user's attention is drawn to the risks associated     */
/* with loading, using, modifying and/or developing or reproducing the        */
/* software by the user in light of its specific status of free software,     */
/* that may mean that it is complicated to manipulate, and that also          */
/* therefore means that it is reserved for developers and experienced         */
/* professionals having in-depth computer knowledge. Users are therefore      */
/* encouraged to load and test the software's suitability as regards          */
/* their requirements in conditions enabling the security of their            */
/* systems and/or data to be ensured and, more generally, to use and          */
/* operate it in the same conditions as regards security.                     */
/*                                                                            */
/* The fact that you are presently reading this means that you have had       */
/* knowledge of the CeCILL-C license and that you accept its terms.           */
/******************************************************************************/

#ifndef HIPS_WRAPPER_H
#define HIPS_WRAPPER_H


#include "block.h"
#include "db_struct.h"
#include "phidal_parallel.h"
#include "db_parallel.h"

#define HIPS_INFOINTNBR 50
#define HIPS_INFOREALNBR 50

struct HIPS_Context{
  PhidalHID BL;
  PhidalDistrHID DBL;
  PhidalOptions options;
  SymbolMatrix *symbmtx, *symbloc;
  PhidalDistrMatrix A;
  /*  PhidalDistrMatrix T;*/

  PhidalDistrPrec Pscal;
  DBDistrPrec Pblock;
  DBPrec      PblockSEQ;
  int_t idmax;
  mpi_t nproc;
  mpi_t proc_id;
  dim_t domsize;
  flag_t partition_type;
  flag_t improve_partition;
  flag_t state;
  flag_t flagrhs;  /** new RHS ? **/
  flag_t flagmat;  /** PhidalMatrix matrix initialized ? **/
  flag_t precond;  /** PhidalMatrix matrix has allready been precond ? **/
  flag_t donotprecond; /** ==1 disable the preconditioning when new matrix coefficient are entered **/
  flag_t assemb_sym; /** use in AssemblyBegin **/
  flag_t hybrid;  /** 0->iterative 1->hybrid **/

  dim_t tagnbr;
  dim_t *perm, *iperm;


  flag_t reorder;
  dim_t ndom;
  dim_t *mapptr;
  dim_t *mapp;
  dim_t ng; /** number of nodes **/
  dim_t na; /** number of unknowns nodes * dof **/
  INTL *ig;
  dim_t *jg;
  COEF *a;

  dim_t *block_psetindex;
  mpi_t *block_pset;

  mpi_t master;
  flag_t coarse_grid;
  flag_t check_graph;
  flag_t check_matrix;
  MPI_Comm mpicom;
  flag_t graph_sym;

  flag_t dumpcsr;

  /*** New Interface ***/
  dim_t ln;
  dim_t dof;
  COEF *x, *b, *t;

  flag_t mode;
  flag_t job, job2;
  LONG ind;
  INTL edgenbr;
  INTL nnz;
  flag_t numflag;
  dim_t *ii;
  dim_t *jj;
  COEF *val;

  /** for grid test **/
  dim_t nc; /** grid size (= one dimension of the square grid) **/
  flag_t cube;

  REAL inforeal[HIPS_INFOREALNBR];
  INTL infoint[HIPS_INFOINTNBR];
};

typedef struct HIPS_Context HIPS_Context;


enum state {INIT=0, GRAPH_BEGIN, GRAPH_END, HIERARCH, SETUP, ASSEMBLY_BEGIN, COEFFICIENT, PRECOND}; 


void PHIDAL_WriteDistrHID(FILE *fp, PhidalDistrHID *DBL);
void PHIDAL_LoadDistrHID(FILE *fp, PhidalDistrHID *DBL);
void PHIDAL_WriteHID(FILE *fp, PhidalHID *BL);
void PHIDAL_LoadHID(FILE *fp, PhidalHID *BL);
void PHIDAL_WriteSymbolMatrix(FILE *fp, SymbolMatrix *symbmtx);
void PHIDAL_LoadSymbolMatrix(FILE *fp, SymbolMatrix* symbmtx);
void PHIDAL_WriteIperm(FILE *fp, dim_t n, dim_t *iperm);
void PHIDAL_LoadIperm(FILE *fp, dim_t n, dim_t **pt_iperm);
int_t PHIDAL_HIDSize(PhidalHID *BL);
INTL* PHIDAL_SerializeHID(INTL *buf, PhidalHID *BL);
INTL* PHIDAL_UnSerializeHID(INTL *buf, PhidalHID *BL);
int_t PHIDAL_SymbolMatrixSize(SymbolMatrix *symbmtx);
INTL* PHIDAL_SerializeSymbolMatrix(INTL *buf, SymbolMatrix *symbmtx);
INTL* PHIDAL_UnSerializeSymbolMatrix(INTL *buf, SymbolMatrix** symbptr);
dim_t PHIDAL_IpermSize(dim_t n, dim_t *iperm);
INTL* PHIDAL_SerializeIperm(INTL *buf, dim_t n, dim_t *iperm);
INTL* PHIDAL_UnSerializeIperm(INTL *buf, dim_t n, dim_t **pt_iperm);



INTS HIPS_Context_Reinit(HIPS_Context *context);
INTS HIPS_Context_Init(HIPS_Context *context);

INTS HIPS_ParallelSetup(INTS id);

INTS HIPS_Precond(INTS id);
INTS HIPS_Solve(INTS id);
INTS HIPS_GraphBuildHID(INTS id);

INTS HIPS_GetGlobalPartitionNbr(INTS id, INTS *domnbr, INTS *partnbr, INTS *domkeynbr);
INTS HIPS_GetGlobalPartition(INTS id, INTS *iperm, INTS *parttab, INTS *domkeyindex, INTS *domkey);
INTS HIPS_GetLocalPartitionNbr(INTS id, INTS *domnbr, INTS *unknownnbr, INTS *partnbr, INTS *partkeynbr);
INTS HIPS_GetLocalPartition(INTS id, INTS *unknownlist,  INTS *parttab, INTS *prockeyindex, INTS *prockey);

INTS Read_options(char *filename,  char *matrix, INTS *sym_pattern, INTS *sym_matrix, char* rhs, INTS* method, INTS *options_int, REAL *options_double);
#endif
