/******************************************************************************/
/* Copyright 2008-2009                                                        */
/*                                                                            */
/* File          : macro.h                                                    */
/*                                                                            */
/* Description   : This file is part of HIPS : Hierarchical Iterative         */
/*                 Parallel Solver (http://hips.gforge.inria.fr/).            */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/* License       : CeCILL-C                                                   */
/*                                                                            */
/* This software is governed by the CeCILL-C license under French law and     */
/* abiding by the rules of distribution of free software. You can use,        */
/* modify and/or redistribute the software under the terms of the             */
/* CeCILL-C license as circulated by CEA, CNRS and INRIA at the following     */
/* URL: "http://www.cecill.info".                                             */
/*                                                                            */
/* As a counterpart to the access to the source code and rights to copy,      */
/* modify and redistribute granted by the license, users are provided         */
/* only with a limited warranty and the software's author, the holder of      */
/* the economic rights, and the successive licensors have only limited        */
/* liability.                                                                 */
/*                                                                            */
/* In this respect, the user's attention is drawn to the risks associated     */
/* with loading, using, modifying and/or developing or reproducing the        */
/* software by the user in light of its specific status of free software,     */
/* that may mean that it is complicated to manipulate, and that also          */
/* therefore means that it is reserved for developers and experienced         */
/* professionals having in-depth computer knowledge. Users are therefore      */
/* encouraged to load and test the software's suitability as regards          */
/* their requirements in conditions enabling the security of their            */
/* systems and/or data to be ensured and, more generally, to use and          */
/* operate it in the same conditions as regards security.                     */
/*                                                                            */
/* The fact that you are presently reading this means that you have had       */
/* knowledge of the CeCILL-C license and that you accept its terms.           */
/******************************************************************************/

#ifndef MACRO_H
#define MACRO_H

#define CMD_(type,name) type ## _ ## name
#define CMD(type,name) CMD_(type,name) 

#define _CMD_2(arg) arg /* utile ? */
#define _CMD_(type,name) _CMD_2(_ ## type ## _ ## name)
#define _CMD(type,name) _CMD_(type,name) 


#if defined(SYMMETRIC)
#define CMDu_(type,name) CMD_(type, name)
#else
#define CMDu_(type,name) type ## _ ## name ## u
#endif
#define CMDu(type,name) CMDu_(type,name) 

/* tmp */
#if defined(SYMMETRIC)
#define CMDus_(type,name) CMD_(type, name)
#else
#define CMDus_(type,name) type ## _ ## name ## _Unsym
#endif
#define CMDus(type,name) CMDus_(type,name) 

#ifndef PARALLEL

#define _DBDistrPrec        DBPrec
#define _DBDistrMatrix      DBMatrix
#define _PhidalDistrPrec    PhidalPrec
#define _PhidalDistrMatrix  PhidalMatrix
#define _PhidalDistrHID     PhidalHID
#define _DBL                BL
#define _DBPrecMem          DBPrecMem
#define _UDBDistrMatrix     UDBMatrix
#define _UPhidalDistrMatrix UPhidalMatrix

#define S(A) A
#define LHID(A) A

#else /* PARALLEL */

#include <mpi.h>
#include "phidal_parallel.h"
#include "db_parallel.h"

#define _DBDistrPrec        DBDistrPrec
#define _DBDistrMatrix      DBDistrMatrix
#define _PhidalDistrPrec    PhidalDistrPrec
#define _PhidalDistrMatrix  PhidalDistrMatrix
#define _PhidalDistrHID     PhidalDistrHID
#define _DBL                DBL
#define _DBPrecMem          DBDistrPrecMem
#define _UDBDistrMatrix     UDBDistrMatrix
#define _UPhidalDistrMatrix UPhidalDistrMatrix

#define S(A) (&A->M)
#define LHID(A) (&A->LHID)

#endif /* PARALLEL */

#endif /* MACRO_H */
