/******************************************************************************/
/* Copyright 2008-2009                                                        */
/*                                                                            */
/* File          : matrix.h                                                   */
/*                                                                            */
/* Description   : This file is part of HIPS : Hierarchical Iterative         */
/*                 Parallel Solver (http://hips.gforge.inria.fr/).            */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/* License       : CeCILL-C                                                   */
/*                                                                            */
/* This software is governed by the CeCILL-C license under French law and     */
/* abiding by the rules of distribution of free software. You can use,        */
/* modify and/or redistribute the software under the terms of the             */
/* CeCILL-C license as circulated by CEA, CNRS and INRIA at the following     */
/* URL: "http://www.cecill.info".                                             */
/*                                                                            */
/* As a counterpart to the access to the source code and rights to copy,      */
/* modify and redistribute granted by the license, users are provided         */
/* only with a limited warranty and the software's author, the holder of      */
/* the economic rights, and the successive licensors have only limited        */
/* liability.                                                                 */
/*                                                                            */
/* In this respect, the user's attention is drawn to the risks associated     */
/* with loading, using, modifying and/or developing or reproducing the        */
/* software by the user in light of its specific status of free software,     */
/* that may mean that it is complicated to manipulate, and that also          */
/* therefore means that it is reserved for developers and experienced         */
/* professionals having in-depth computer knowledge. Users are therefore      */
/* encouraged to load and test the software's suitability as regards          */
/* their requirements in conditions enabling the security of their            */
/* systems and/or data to be ensured and, more generally, to use and          */
/* operate it in the same conditions as regards security.                     */
/*                                                                            */
/* The fact that you are presently reading this means that you have had       */
/* knowledge of the CeCILL-C license and that you accept its terms.           */
/******************************************************************************/

#ifndef MATRIX_H
#define MATRIX_H

typedef struct PhidalMatrix_ PhidalMatrix;
struct UPhidalMatrix_ {
  PhidalMatrix* L;
  PhidalMatrix* U;
};
typedef struct UPhidalMatrix_ UPhidalMatrix;

typedef struct PhidalDistrMatrix_ PhidalDistrMatrix;
struct UPhidalDistrMatrix_ {
  PhidalDistrMatrix* L;
  PhidalDistrMatrix* U;
};
typedef struct UPhidalDistrMatrix_ UPhidalDistrMatrix;

typedef struct DBMatrix_ DBMatrix;
struct UDBMatrix_ {
  DBMatrix* L;
  DBMatrix* U;
};
typedef struct UDBMatrix_ UDBMatrix;

typedef struct DBDistrMatrix_ DBDistrMatrix;
struct UDBDistrMatrix_ {
  DBDistrMatrix* L;
  DBDistrMatrix* U;
};
typedef struct UDBDistrMatrix_ UDBDistrMatrix;

#ifndef PARALLEL
#define D(arg) arg
#else
#define D(arg) d ## arg
#endif

#define M_MALLOC(mtx, t)  { mtx = (Matrix*)malloc(sizeof(Matrix)); mtx->type = t; }
#define M_CLEAN(mtx)    { if (mtx != NULL) { free(mtx); mtx=NULL; } }

#define M_BLOCK(mtx) mtx->m.D(b)
#define M_SCAL(mtx)  mtx->m.D(s)

enum MatrixType {SCAL, BLOCK, dSCAL, dBLOCK};

typedef union  {
  UPhidalMatrix       s;
  UPhidalDistrMatrix ds;
  UDBMatrix           b;
  UDBDistrMatrix     db;
} Matrix__;

typedef struct Matrix_ Matrix;
struct Matrix_ {
  flag_t   type;
  Matrix__ m;
};

#endif /*MATRIX_H*/
