/******************************************************************************/
/* Copyright 2008-2009 CNRS                                                   */
/*                                                                            */
/* File          : prec.h                                                     */
/*                                                                            */
/* Description   : This file is part of HIPS : Hierarchical Iterative         */
/*                 Parallel Solver (http://hips.gforge.inria.fr/).            */
/*                                                                            */
/* Authors       : J. GAIDAMOUR                                               */
/******************************************************************************/

/******************************************************************************/
/* License       : CeCILL-C                                                   */
/*                                                                            */
/* This software is governed by the CeCILL-C license under French law and     */
/* abiding by the rules of distribution of free software. You can use,        */
/* modify and/or redistribute the software under the terms of the             */
/* CeCILL-C license as circulated by CEA, CNRS and INRIA at the following     */
/* URL: "http://www.cecill.info".                                             */
/*                                                                            */
/* As a counterpart to the access to the source code and rights to copy,      */
/* modify and redistribute granted by the license, users are provided         */
/* only with a limited warranty and the software's author, the holder of      */
/* the economic rights, and the successive licensors have only limited        */
/* liability.                                                                 */
/*                                                                            */
/* In this respect, the user's attention is drawn to the risks associated     */
/* with loading, using, modifying and/or developing or reproducing the        */
/* software by the user in light of its specific status of free software,     */
/* that may mean that it is complicated to manipulate, and that also          */
/* therefore means that it is reserved for developers and experienced         */
/* professionals having in-depth computer knowledge. Users are therefore      */
/* encouraged to load and test the software's suitability as regards          */
/* their requirements in conditions enabling the security of their            */
/* systems and/or data to be ensured and, more generally, to use and          */
/* operate it in the same conditions as regards security.                     */
/*                                                                            */
/* The fact that you are presently reading this means that you have had       */
/* knowledge of the CeCILL-C license and that you accept its terms.           */
/******************************************************************************/


#ifndef PREC_H
#define PREC_H

#include "matrix.h"

typedef struct PhidalPrec_ PhidalPrec;
typedef struct PhidalDistrPrec_ PhidalDistrPrec;

/* /!\ #define PARALLEL must be set before #include "prec.h" */





/* #include "phidal_struct.h" */
/* #include <mpi.h> */
/* #include "phidal_parallel_struct.h" */
/* #include "db_parallel_struct.h" */

/* #include "phidal_parallel_struct_sub.h" */
/* #include "db_parallel_struct_sub.h" */

/* #include "solver.h" */

typedef struct Prec_  Prec;
struct Prec_ {
  flag_t type; /* temporaire, il n'en restera plus qu'un ... */

  flag_t symmetric; 
  dim_t dim;
  int_t levelnum;
  int_t forwardlev;

  Matrix* LU;

  Matrix* EF;

  Matrix* EF_DB; /* tmp, a suppr */

  Matrix* B;

  Matrix* S;

  Prec *prevprec;
  Prec *nextprec;

  flag_t schur_method;

  PrecInfo* info;

  /* scalar : */
  flag_t pivoting;  /** 1 := column pivoting in ILUTP */
  dim_t *permtab;

  /*tmp*/
  PhidalMatrix *phidalS;
  PhidalPrec   *phidalPrec;

};

/* TMP */
typedef struct DistrPrec_  DistrPrec;
struct DistrPrec_ {
  flag_t type; /* temporaire, il n'en restera plus qu'un ... */

  flag_t symmetric; 
  dim_t dim;
  int_t levelnum;
  int_t forwardlev;

  Matrix* LU;

  Matrix* EF;

  Matrix* EF_DB; /* tmp, a suppr */

  Matrix* B;

  Matrix* S;

  DistrPrec *prevprec;
  DistrPrec *nextprec;

  flag_t schur_method;

  PrecInfo* info;

  /* scalar : */
  flag_t pivoting;  /** 1 := column pivoting in ILUTP */
  dim_t *permtab;

  /*tmp*/
  PhidalDistrMatrix *phidalS;
  PhidalDistrPrec   *phidalPrec;

};


#undef PREC_L
#undef PREC_U
#undef PREC_EDB
#undef PREC_FDB
#undef PREC_SL
#undef PREC_SU

#define PREC_LU_BLOCK(P) (P)->M_BLOCK(LU)
#define PREC_L_BLOCK(P)  (P)->M_BLOCK(LU).L
#define PREC_U_BLOCK(P)  (P)->M_BLOCK(LU).U

#define PREC_LU_SCAL(P) (P)->M_SCAL(LU)
#define PREC_L_SCAL(P)  (P)->M_SCAL(LU).L
#define PREC_U_SCAL(P)  (P)->M_SCAL(LU).U


/** OIMBE  A VIRER*/
#define PREC_EF(P) (P)->M_SCAL(EF)
#define PREC_E(P)  (P)->M_SCAL(EF).L
#define PREC_F(P)  (P)->M_SCAL(EF).U
/*** ===> Prendre des notations coherentes : ***/
#define PREC_EF_SCAL(P) (P)->M_SCAL(EF)
#define PREC_E_SCAL(P)  (P)->M_SCAL(EF).L
#define PREC_F_SCAL(P)  (P)->M_SCAL(EF).U


/** OIMBE  A VIRER*/
#define PREC_EFDB(P) (P)->M_BLOCK(EF_DB)
#define PREC_EDB(P) (P)->M_BLOCK(EF_DB).L
#define PREC_FDB(P) (P)->M_BLOCK(EF_DB).U
/*** ===> Prendre des notations coherentes : ***/
#define PREC_EF_BLOCK(P) (P)->M_BLOCK(EF_DB)
#define PREC_E_BLOCK(P) (P)->M_BLOCK(EF_DB).L
#define PREC_F_BLOCK(P) (P)->M_BLOCK(EF_DB).U

#define PREC_S_BLOCK(P)  (P)->M_BLOCK(S)
#define PREC_SL_BLOCK(P) (P)->M_BLOCK(S).L
#define PREC_SU_BLOCK(P) (P)->M_BLOCK(S).U

#define PREC_S_SCAL(P)  (P)->M_SCAL(S).L

/** OIMBE  A VIRER*/
#define PREC_B(P)  (P)->M_SCAL(B).L
/** Unification **/
#define PREC_B_SCAL(P)  (P)->M_SCAL(B).L

#ifndef PARALLEL

#if defined(SYMMETRIC)
#define _MATRIX_MLILUPrec     MATRIX_MLICCPrec
#elif defined(UNSYMMETRIC)
#define _MATRIX_MLILUPrec     MATRIX_MLILUPrec
#endif

#else /* PARALLEL */

#if defined(SYMMETRIC)
#define _MATRIX_MLILUPrec     DISTRMATRIX_MLICCPrec
#elif defined(UNSYMMETRIC)
#define _MATRIX_MLILUPrec     DISTRMATRIX_MLILUPrec
#endif

#endif /* PARALLEL */


#endif


