/******************************************************************************/
/* Copyright 2008-2009 INRIA                                                  */
/*                                                                            */
/* File          : queue.h                                                    */
/*                                                                            */
/* Description   : This file is part of HIPS : Hierarchical Iterative         */
/*                 Parallel Solver (http://hips.gforge.inria.fr/).            */
/*                                                                            */
/* Authors       : P. HENON                                                   */
/******************************************************************************/

/******************************************************************************/
/* License       : CeCILL-C                                                   */
/*                                                                            */
/* This software is governed by the CeCILL-C license under French law and     */
/* abiding by the rules of distribution of free software. You can use,        */
/* modify and/or redistribute the software under the terms of the             */
/* CeCILL-C license as circulated by CEA, CNRS and INRIA at the following     */
/* URL: "http://www.cecill.info".                                             */
/*                                                                            */
/* As a counterpart to the access to the source code and rights to copy,      */
/* modify and redistribute granted by the license, users are provided         */
/* only with a limited warranty and the software's author, the holder of      */
/* the economic rights, and the successive licensors have only limited        */
/* liability.                                                                 */
/*                                                                            */
/* In this respect, the user's attention is drawn to the risks associated     */
/* with loading, using, modifying and/or developing or reproducing the        */
/* software by the user in light of its specific status of free software,     */
/* that may mean that it is complicated to manipulate, and that also          */
/* therefore means that it is reserved for developers and experienced         */
/* professionals having in-depth computer knowledge. Users are therefore      */
/* encouraged to load and test the software's suitability as regards          */
/* their requirements in conditions enabling the security of their            */
/* systems and/or data to be ensured and, more generally, to use and          */
/* operate it in the same conditions as regards security.                     */
/*                                                                            */
/* The fact that you are presently reading this means that you have had       */
/* knowledge of the CeCILL-C license and that you accept its terms.           */
/******************************************************************************/



#ifndef QUEUE_H
#define QUEUE_H


#include "hips_define.h"
/*#ifndef INT
#define INTL int
#endif */
#ifndef FLOAT
#define FLOAT float
#endif
/*
**  The type and structure definitions.
*/

typedef struct Queue_ {
  dim_t        size;                  /*+ Allocated memory size             +*/ 
  dim_t        used;                  /*+ Number of element in the queue    +*/
  dim_t    *   elttab;                /*+ Array of the element              +*/
  FLOAT  *   keytab;                /*+ Array of keys                     +*/
  dim_t    *   keytab2;               /*+ Another array of keys             +*/
} Queue;


#define static

int     queueInit       (Queue *, dim_t size);
void    queueExit       (Queue *);
Queue * queueCopy       (Queue *dst, Queue *src);
void    queueAdd        (Queue *, dim_t, FLOAT);
void    queueAdd2       (Queue *, dim_t, FLOAT, dim_t);
dim_t     queueGet        (Queue *);
void    queueGet2       (Queue *, dim_t *, FLOAT *);
void    queueGet3       (Queue *, dim_t *, dim_t *);
dim_t     queueSize       (Queue *);
void    queueClear      (Queue *);
dim_t     queueRead       (Queue *);
dim_t     queueReadKey    (Queue *);
static int compWith2keys(Queue *, dim_t, dim_t);
#undef static
#endif


#ifndef HEAP_H
#define HEAP_H

/*
**  The type and structure definitions.
*/

typedef struct Heap_ {
  dim_t        size;                  /*+ Allocated memory size             +*/ 
  dim_t        used;                  /*+ Number of element in the queue    +*/
  dim_t    *   elttab;                /*+ Array of the element              +*/
} Heap;


/*#define static*/

int     Heap_Init       (Heap *, dim_t size);
void    Heap_Exit       (Heap *);
Heap  * Heap_Copy       (Heap *dst, Heap *src);
void    Heap_Add        (Heap *, dim_t);
dim_t     Heap_Get        (Heap *);
dim_t     Heap_Size       (Heap *);
void    Heap_Clear      (Heap *);
dim_t     Heap_Read       (Heap *);


#endif
