/******************************************************************************/
/* Copyright 2008-2009 INRIA                                                  */
/*                                                                            */
/* File          : scotch_metis_wrapper.h                                     */
/*                                                                            */
/* Description   : This file is part of HIPS : Hierarchical Iterative         */
/*                 Parallel Solver (http://hips.gforge.inria.fr/).            */
/*                                                                            */
/* Authors       : P. HENON                                                   */
/******************************************************************************/

/******************************************************************************/
/* License       : CeCILL-C                                                   */
/*                                                                            */
/* This software is governed by the CeCILL-C license under French law and     */
/* abiding by the rules of distribution of free software. You can use,        */
/* modify and/or redistribute the software under the terms of the             */
/* CeCILL-C license as circulated by CEA, CNRS and INRIA at the following     */
/* URL: "http://www.cecill.info".                                             */
/*                                                                            */
/* As a counterpart to the access to the source code and rights to copy,      */
/* modify and redistribute granted by the license, users are provided         */
/* only with a limited warranty and the software's author, the holder of      */
/* the economic rights, and the successive licensors have only limited        */
/* liability.                                                                 */
/*                                                                            */
/* In this respect, the user's attention is drawn to the risks associated     */
/* with loading, using, modifying and/or developing or reproducing the        */
/* software by the user in light of its specific status of free software,     */
/* that may mean that it is complicated to manipulate, and that also          */
/* therefore means that it is reserved for developers and experienced         */
/* professionals having in-depth computer knowledge. Users are therefore      */
/* encouraged to load and test the software's suitability as regards          */
/* their requirements in conditions enabling the security of their            */
/* systems and/or data to be ensured and, more generally, to use and          */
/* operate it in the same conditions as regards security.                     */
/*                                                                            */
/* The fact that you are presently reading this means that you have had       */
/* knowledge of the CeCILL-C license and that you accept its terms.           */
/******************************************************************************/

#ifndef SCOTCH_METIS_WRAPPER_H
#define SCOTCH_METIS_WRAPPER_H

/*** wrapper for scotch and metis (solve the problem of int types 32/64) ***/
#ifdef SCOTCH_PART
#define restrict
#include "scotch.h"

int SCOTCH_graphOrder_WRAPPER( SCOTCH_Graph *  graphdatptr,  SCOTCH_Strat *  stratdatptr, dim_t n, dim_t *perm, dim_t *rperm, dim_t *cblknbr, dim_t *rangtab, dim_t *treetab);

int SCOTCH_graphBuild_WRAPPER(SCOTCH_Graph *  grafdat, flag_t numflag, dim_t n, INTL *ia,  INTL *vwgt, dim_t *ja, INTL *ewgt);

int SCOTCH_graphPart_WRAPPER(dim_t n, SCOTCH_Graph *  grafdat, dim_t ndom,  SCOTCH_Strat *  grafstrat, mpi_t *node2dom);


int SCOTCH_graphPartOvl_WRAPPER( dim_t n, SCOTCH_Graph *  grafdat, dim_t ndom,  SCOTCH_Strat *  grafstrat, mpi_t *node2dom);

#else

#include "metis.h"

void METIS_NodeWND_WRAPPER(dim_t n, INTL *ia, dim_t *ja, INTL *wgt, flag_t numflag, int *options, dim_t *iperm, dim_t *perm);
void METIS_NodeND_WRAPPER(dim_t n,  INTL *ia, dim_t *ja, flag_t numflag, int *options, dim_t *rperm, dim_t *perm);
void METIS_PartGraphKway_WRAPPER(dim_t n, INTL *ia, dim_t *ja, INTL *vwgt, INTL *ewgt, flag_t wgtflag, flag_t numflag, int ndom, int *options, int *edgecutptr, dim_t *mpi2proc);
void METIS_PartGraphRecursive_WRAPPER(dim_t n, INTL *ia, dim_t *ja, INTL *vwgt, INTL *ewgt, flag_t wgtflag, flag_t numflag, int ndom, int *options, int *edgecutptr, mpi_t *dom2proc);
void METIS_PartGraphVKway_WRAPPER(dim_t n, INTL *ia, dim_t *ja, INTL *vwgt, INTL *ewgt, flag_t wgtflag, flag_t numflag, int ndom, int *options, int *edgecutptr, mpi_t *dom2proc);
#endif

#endif
