/******************************************************************************/
/* Copyright 2008-2009 CNRS, INRIA                                            */
/*                                                                            */
/* File          : symbol.h                                                   */
/*                                                                            */
/* Description   : This file is part of HIPS : Hierarchical Iterative         */
/*                 Parallel Solver (http://hips.gforge.inria.fr/).            */
/*                                                                            */
/* Authors       : J. GAIDAMOUR and P. HENON                                  */
/******************************************************************************/

/******************************************************************************/
/* License       : CeCILL-C                                                   */
/*                                                                            */
/* This software is governed by the CeCILL-C license under French law and     */
/* abiding by the rules of distribution of free software. You can use,        */
/* modify and/or redistribute the software under the terms of the             */
/* CeCILL-C license as circulated by CEA, CNRS and INRIA at the following     */
/* URL: "http://www.cecill.info".                                             */
/*                                                                            */
/* As a counterpart to the access to the source code and rights to copy,      */
/* modify and redistribute granted by the license, users are provided         */
/* only with a limited warranty and the software's author, the holder of      */
/* the economic rights, and the successive licensors have only limited        */
/* liability.                                                                 */
/*                                                                            */
/* In this respect, the user's attention is drawn to the risks associated     */
/* with loading, using, modifying and/or developing or reproducing the        */
/* software by the user in light of its specific status of free software,     */
/* that may mean that it is complicated to manipulate, and that also          */
/* therefore means that it is reserved for developers and experienced         */
/* professionals having in-depth computer knowledge. Users are therefore      */
/* encouraged to load and test the software's suitability as regards          */
/* their requirements in conditions enabling the security of their            */
/* systems and/or data to be ensured and, more generally, to use and          */
/* operate it in the same conditions as regards security.                     */
/*                                                                            */
/* The fact that you are presently reading this means that you have had       */
/* knowledge of the CeCILL-C license and that you accept its terms.           */
/******************************************************************************/


#ifndef SYMBOL_H
#define SYMBOL_H

#define SYMBOL_VERSION              1

/** Need to include base.h here because it contains essl.h
    and there is trouble on AIX with "stride" defined as a
    macro if essl.h is not include **/
#include "hips_define.h"
#include "type.h"
#include "base.h"

/*
**  The type and structure definitions.
*/

/*+ The column block structure. +*/

typedef struct SymbolCCblk_ {
  dim_t                       fcolnum;              /*+ First column index               +*/
  dim_t                       lcolnum;              /*+ Last column index (inclusive)    +*/ /* TODO : utilité de cela ? */
} SymbolCCblk;


typedef struct SymbolBCblk_ {
  INTL                       fbloknum;             /*+ First block in column (diagonal) +*/
  INTL                       lbloknum;             /*+ Last  block in column            +*/
} SymbolBCblk;

/*+ The column block structure. +*/
typedef struct SymbolBlok_ {
  dim_t                     frownum;              /*+ First row index            +*/
  dim_t                     lrownum;              /*+ Last row index (inclusive) +*/
  dim_t                     cblknum;              /*+ Facing column block        +*/
} SymbolBlok;

/*+ The symbolic block matrix. +*/
typedef struct SymbolMatrix_ {
  dim_t                     cblknbr;              /*+ Number of column blocks           +*/

  SymbolCCblk *             ccblktab;             /*+ Array of column blocks            +*/
  SymbolBCblk *             bcblktab;             /*+ Array of column blocks            +*/
  blas_t *                  stride; 
  blas_t *                  hdim;

  INTL                       bloknbr;              /*+ Number of blocks                  +*/
  SymbolBlok *              bloktab;              /*+ Array of blocks [based]           +*/

  dim_t                     nodenbr;              /*+ Number of nodes in matrix         +*/

  dim_t                     tli;
  dim_t                     facedecal;
  dim_t                     cblktlj; /* todo : n'existe qu'apres SymbolMatrixCut1 */
  flag_t                    virtual;
} SymbolMatrix;


/* // Interval */
typedef struct BlokStruct_ {
  dim_t                     frownum;              /*+ First column index               +*/
  dim_t                     lrownum;              /*+ Last column index (inclusive)    +*/
} BlokStruct;

typedef struct CblkStruct_ {
  INTL bloknbr;                                       /* Number of blocks    */
  BlokStruct* bloktab;                              /* Array of blocks     */
} CblkStruct;

typedef CblkStruct MatrixStruct;                 /* Array of column blocks */





#endif
